@@ -4,7 +4,8 @@
 #' Gregorian year, month, day, and time of day components.
 #'
 #' @details
-#' Fields are recycled against each other.
+#' Fields are recycled against each other using
+#' [tidyverse recycling rules][vctrs::vector_recycling_rules].
 #'
 #' Fields are collected in order until the first `NULL` field is located. No
 #' fields after the first `NULL` field are used.
@@ -100,7 +101,8 @@ year_month_day <- function(year,
     precision <- PRECISION_SECOND
     fields <- list(year = year, month = month, day = day, hour = hour, minute = minute, second = second)
   } else {
-    precision <- calendar_validate_subsecond_precision(subsecond_precision)
+    calendar_check_subsecond_precision(subsecond_precision)
+    precision <- precision_to_integer(subsecond_precision)
     fields <- list(year = year, month = month, day = day, hour = hour, minute = minute, second = second, subsecond = subsecond)
   }
 
@@ -111,13 +113,34 @@ year_month_day <- function(year,
     last <- FALSE
   }
 
-  fields <- vec_recycle_common(!!!fields)
   fields <- vec_cast_common(!!!fields, .to = integer())
 
-  fields <- collect_year_month_day_fields(fields, precision)
+  if (precision >= PRECISION_YEAR) {
+    check_between_year(fields$year, arg = "year")
+  }
+  if (precision >= PRECISION_MONTH) {
+    check_between_month(fields$month, arg = "month")
+  }
+  if (precision >= PRECISION_DAY) {
+    check_between_day_of_month(fields$day, arg = "day")
+  }
+  if (precision >= PRECISION_HOUR) {
+    check_between_hour(fields$hour, arg = "hour")
+  }
+  if (precision >= PRECISION_MINUTE) {
+    check_between_minute(fields$minute, arg = "minute")
+  }
+  if (precision >= PRECISION_SECOND) {
+    check_between_second(fields$second, arg = "second")
+  }
+  if (precision > PRECISION_SECOND) {
+    check_between_subsecond(fields$subsecond, precision, arg = "subsecond")
+  }
+
+  fields <- vec_recycle_common(!!!fields)
+  fields <- df_list_propagate_missing(fields)
 
   names <- NULL
-
   out <- new_year_month_day_from_fields(fields, precision, names)
 
   if (last) {
@@ -261,18 +284,14 @@ year_month_day_parse <- function(x,
     abort("`x` must be a character vector.")
   }
 
-  precision <- validate_precision_string(precision)
-  if (!year_month_day_is_valid_precision(precision)) {
-    abort("`precision` must be a valid precision for 'year_month_day'.")
-  }
+  year_month_day_check_precision(precision)
+  precision <- precision_to_integer(precision)
 
   if (is_null(format)) {
     format <- year_month_day_format(precision)
   }
 
-  if (!is_clock_locale(locale)) {
-    abort("`locale` must be a 'clock_locale' object.")
-  }
+  check_clock_locale(locale)
 
   labels <- locale$labels
   mark <- locale$decimal_mark
@@ -364,14 +383,12 @@ vec_cast.clock_year_month_day.clock_year_month_day <- function(x, to, ...) {
 # ------------------------------------------------------------------------------
 
 #' @export
-calendar_is_valid_precision.clock_year_month_day <- function(x, precision) {
-  year_month_day_is_valid_precision(precision)
+calendar_is_precision.clock_year_month_day <- function(x, precision) {
+  year_month_day_is_precision(precision)
 }
 
-year_month_day_is_valid_precision <- function(precision) {
-  if (!is_valid_precision(precision)) {
-    FALSE
-  } else if (precision == PRECISION_YEAR || precision == PRECISION_MONTH) {
+year_month_day_is_precision <- function(precision) {
+  if (precision == PRECISION_YEAR || precision == PRECISION_MONTH) {
     TRUE
   } else if (precision >= PRECISION_DAY && precision <= PRECISION_NANOSECOND) {
     TRUE
@@ -380,21 +397,64 @@ year_month_day_is_valid_precision <- function(precision) {
   }
 }
 
+year_month_day_check_precision <- function(x,
+                                           ...,
+                                           arg = caller_arg(x),
+                                           call = caller_env()) {
+  check_precision(
+    x = x,
+    values = year_month_day_precision_names(),
+    arg = arg,
+    call = call
+  )
+}
+
+year_month_day_precision_names <- function() {
+  c("year", "month", "day", precision_time_names())
+}
+
 # ------------------------------------------------------------------------------
 
 #' @export
 invalid_detect.clock_year_month_day <- function(x) {
-  invalid_detect_year_month_day_cpp(x, calendar_precision_attribute(x))
+  precision <- calendar_precision_attribute(x)
+
+  if (precision < PRECISION_DAY) {
+    rep_along(x, FALSE)
+  } else {
+    year <- field_year(x)
+    month <- field_month(x)
+    day <- field_day(x)
+    invalid_detect_year_month_day_cpp(year, month, day)
+  }
 }
 
 #' @export
 invalid_any.clock_year_month_day <- function(x) {
-  invalid_any_year_month_day_cpp(x, calendar_precision_attribute(x))
+  precision <- calendar_precision_attribute(x)
+
+  if (precision < PRECISION_DAY) {
+    FALSE
+  } else {
+    year <- field_year(x)
+    month <- field_month(x)
+    day <- field_day(x)
+    invalid_any_year_month_day_cpp(year, month, day)
+  }
 }
 
 #' @export
 invalid_count.clock_year_month_day <- function(x) {
-  invalid_count_year_month_day_cpp(x, calendar_precision_attribute(x))
+  precision <- calendar_precision_attribute(x)
+
+  if (precision < PRECISION_DAY) {
+    0L
+  } else {
+    year <- field_year(x)
+    month <- field_month(x)
+    day <- field_day(x)
+    invalid_count_year_month_day_cpp(year, month, day)
+  }
 }
 
 #' @export
@@ -402,8 +462,13 @@ invalid_resolve.clock_year_month_day <- function(x, ..., invalid = NULL) {
   check_dots_empty()
   precision <- calendar_precision_attribute(x)
   invalid <- validate_invalid(invalid)
-  fields <- invalid_resolve_year_month_day_cpp(x, precision, invalid)
-  new_year_month_day_from_fields(fields, precision, names(x))
+
+  if (precision < PRECISION_DAY) {
+    x
+  } else {
+    fields <- invalid_resolve_year_month_day_cpp(x, precision, invalid, current_env())
+    new_year_month_day_from_fields(fields, precision, names(x))
+  }
 }
 
 # ------------------------------------------------------------------------------
@@ -452,56 +517,56 @@ get_year.clock_year_month_day <- function(x) {
 #' @rdname year-month-day-getters
 #' @export
 get_month.clock_year_month_day <- function(x) {
-  calendar_require_minimum_precision(x, PRECISION_MONTH, "get_month")
+  calendar_check_minimum_precision(x, PRECISION_MONTH)
   field_month(x)
 }
 
 #' @rdname year-month-day-getters
 #' @export
 get_day.clock_year_month_day <- function(x) {
-  calendar_require_minimum_precision(x, PRECISION_DAY, "get_day")
+  calendar_check_minimum_precision(x, PRECISION_DAY)
   field_day(x)
 }
 
 #' @rdname year-month-day-getters
 #' @export
 get_hour.clock_year_month_day <- function(x) {
-  calendar_require_minimum_precision(x, PRECISION_HOUR, "get_hour")
+  calendar_check_minimum_precision(x, PRECISION_HOUR)
   field_hour(x)
 }
 
 #' @rdname year-month-day-getters
 #' @export
 get_minute.clock_year_month_day <- function(x) {
-  calendar_require_minimum_precision(x, PRECISION_MINUTE, "get_minute")
+  calendar_check_minimum_precision(x, PRECISION_MINUTE)
   field_minute(x)
 }
 
 #' @rdname year-month-day-getters
 #' @export
 get_second.clock_year_month_day <- function(x) {
-  calendar_require_minimum_precision(x, PRECISION_SECOND, "get_second")
+  calendar_check_minimum_precision(x, PRECISION_SECOND)
   field_second(x)
 }
 
 #' @rdname year-month-day-getters
 #' @export
 get_millisecond.clock_year_month_day <- function(x) {
-  calendar_require_precision(x, PRECISION_MILLISECOND, "get_millisecond")
+  calendar_check_exact_precision(x, PRECISION_MILLISECOND)
   field_subsecond(x)
 }
 
 #' @rdname year-month-day-getters
 #' @export
 get_microsecond.clock_year_month_day <- function(x) {
-  calendar_require_precision(x, PRECISION_MICROSECOND, "get_microsecond")
+  calendar_check_exact_precision(x, PRECISION_MICROSECOND)
   field_subsecond(x)
 }
 
 #' @rdname year-month-day-getters
 #' @export
 get_nanosecond.clock_year_month_day <- function(x) {
-  calendar_require_precision(x, PRECISION_NANOSECOND, "get_nanosecond")
+  calendar_check_exact_precision(x, PRECISION_NANOSECOND)
   field_subsecond(x)
 }
 
@@ -570,7 +635,7 @@ set_year.clock_year_month_day <- function(x, value, ...) {
 #' @export
 set_month.clock_year_month_day <- function(x, value, ...) {
   check_dots_empty()
-  calendar_require_minimum_precision(x, PRECISION_YEAR, "set_month")
+  calendar_check_minimum_precision(x, PRECISION_YEAR)
   set_field_year_month_day(x, value, "month")
 }
 
@@ -578,7 +643,7 @@ set_month.clock_year_month_day <- function(x, value, ...) {
 #' @export
 set_day.clock_year_month_day <- function(x, value, ...) {
   check_dots_empty()
-  calendar_require_minimum_precision(x, PRECISION_MONTH, "set_day")
+  calendar_check_minimum_precision(x, PRECISION_MONTH)
   set_field_year_month_day(x, value, "day")
 }
 
@@ -586,7 +651,7 @@ set_day.clock_year_month_day <- function(x, value, ...) {
 #' @export
 set_hour.clock_year_month_day <- function(x, value, ...) {
   check_dots_empty()
-  calendar_require_minimum_precision(x, PRECISION_DAY, "set_hour")
+  calendar_check_minimum_precision(x, PRECISION_DAY)
   set_field_year_month_day(x, value, "hour")
 }
 
@@ -594,7 +659,7 @@ set_hour.clock_year_month_day <- function(x, value, ...) {
 #' @export
 set_minute.clock_year_month_day <- function(x, value, ...) {
   check_dots_empty()
-  calendar_require_minimum_precision(x, PRECISION_HOUR, "set_minute")
+  calendar_check_minimum_precision(x, PRECISION_HOUR)
   set_field_year_month_day(x, value, "minute")
 }
 
@@ -602,7 +667,7 @@ set_minute.clock_year_month_day <- function(x, value, ...) {
 #' @export
 set_second.clock_year_month_day <- function(x, value, ...) {
   check_dots_empty()
-  calendar_require_minimum_precision(x, PRECISION_MINUTE, "set_second")
+  calendar_check_minimum_precision(x, PRECISION_MINUTE)
   set_field_year_month_day(x, value, "second")
 }
 
@@ -610,7 +675,7 @@ set_second.clock_year_month_day <- function(x, value, ...) {
 #' @export
 set_millisecond.clock_year_month_day <- function(x, value, ...) {
   check_dots_empty()
-  calendar_require_any_of_precisions(x, c(PRECISION_SECOND, PRECISION_MILLISECOND), "set_millisecond")
+  calendar_check_exact_precision(x, c(PRECISION_SECOND, PRECISION_MILLISECOND))
   set_field_year_month_day(x, value, "millisecond")
 }
 
@@ -618,7 +683,7 @@ set_millisecond.clock_year_month_day <- function(x, value, ...) {
 #' @export
 set_microsecond.clock_year_month_day <- function(x, value, ...) {
   check_dots_empty()
-  calendar_require_any_of_precisions(x, c(PRECISION_SECOND, PRECISION_MICROSECOND), "set_microsecond")
+  calendar_check_exact_precision(x, c(PRECISION_SECOND, PRECISION_MICROSECOND))
   set_field_year_month_day(x, value, "microsecond")
 }
 
@@ -626,7 +691,7 @@ set_microsecond.clock_year_month_day <- function(x, value, ...) {
 #' @export
 set_nanosecond.clock_year_month_day <- function(x, value, ...) {
   check_dots_empty()
-  calendar_require_any_of_precisions(x, c(PRECISION_SECOND, PRECISION_NANOSECOND), "set_nanosecond")
+  calendar_check_exact_precision(x, c(PRECISION_SECOND, PRECISION_NANOSECOND))
   set_field_year_month_day(x, value, "nanosecond")
 }
 
@@ -639,28 +704,52 @@ set_field_year_month_day <- function(x, value, component) {
   precision_value <- year_month_day_component_to_precision(component)
   precision_out <- precision_common2(precision_fields, precision_value)
 
-  value <- vec_cast(value, integer(), x_arg = "value")
+  names_out <- names(x)
+
+  value <- vec_cast(value, integer())
+  value <- unname(value)
+
+  switch(
+    component,
+    year = check_between_year(value),
+    month = check_between_month(value),
+    day = check_between_day_of_month(value),
+    hour = check_between_hour(value),
+    minute = check_between_minute(value),
+    second = check_between_second(value),
+    millisecond = check_between_subsecond(value, PRECISION_MILLISECOND),
+    microsecond = check_between_subsecond(value, PRECISION_MICROSECOND),
+    nanosecond = check_between_subsecond(value, PRECISION_NANOSECOND),
+    abort("Unknown `component`", .internal = TRUE)
+  )
+
   args <- vec_recycle_common(x = x, value = value)
+  args <- df_list_propagate_missing(args)
   x <- args$x
   value <- args$value
 
-  result <- set_field_year_month_day_cpp(x, value, precision_fields, precision_value)
-  fields <- result$fields
   field <- year_month_day_component_to_field(component)
-  fields[[field]] <- result$value
 
-  new_year_month_day_from_fields(fields, precision_out, names = names(x))
+  out <- vec_unstructure(x)
+  out[[field]] <- value
+
+  new_year_month_day_from_fields(out, precision_out, names = names_out)
 }
 
 set_field_year_month_day_last <- function(x) {
   precision_fields <- calendar_precision_attribute(x)
   precision_out <- precision_common2(precision_fields, PRECISION_DAY)
 
-  result <- set_field_year_month_day_last_cpp(x, precision_fields)
-  fields <- result$fields
-  fields[["day"]] <- result$value
+  names_out <- names(x)
 
-  new_year_month_day_from_fields(fields, precision_out, names = names(x))
+  year <- field_year(x)
+  month <- field_month(x)
+  value <- get_year_month_day_last_cpp(year, month)
+
+  out <- vec_unstructure(x)
+  out[["day"]] <- value
+
+  new_year_month_day_from_fields(out, precision_out, names = names_out)
 }
 
 # ------------------------------------------------------------------------------
@@ -799,9 +888,10 @@ year_month_day_minus_year_month_day <- function(op, x, y, ...) {
 #' Adding a single quarter with `add_quarters()` is equivalent to adding
 #' 3 months.
 #'
-#' `x` and `n` are recycled against each other.
+#' `x` and `n` are recycled against each other using
+#' [tidyverse recycling rules][vctrs::vector_recycling_rules].
 #'
-#' @inheritParams add_years
+#' @inheritParams clock-arithmetic
 #'
 #' @param x `[clock_year_month_day]`
 #'
@@ -844,30 +934,58 @@ add_years.clock_year_month_day <- function(x, n, ...) {
 #' @rdname year-month-day-arithmetic
 #' @export
 add_quarters.clock_year_month_day <- function(x, n, ...) {
-  calendar_require_minimum_precision(x, PRECISION_MONTH, "add_quarters")
+  calendar_check_minimum_precision(x, PRECISION_MONTH)
   year_month_day_plus_duration(x, n, PRECISION_QUARTER)
 }
 
 #' @rdname year-month-day-arithmetic
 #' @export
 add_months.clock_year_month_day <- function(x, n, ...) {
-  calendar_require_minimum_precision(x, PRECISION_MONTH, "add_months")
+  calendar_check_minimum_precision(x, PRECISION_MONTH)
   year_month_day_plus_duration(x, n, PRECISION_MONTH)
 }
 
-year_month_day_plus_duration <- function(x, n, precision_n) {
-  precision_fields <- calendar_precision_attribute(x)
+year_month_day_plus_duration <- function(x,
+                                         n,
+                                         n_precision,
+                                         ...,
+                                         error_call = caller_env()) {
+  check_dots_empty0(...)
 
-  n <- duration_collect_n(n, precision_n)
-  args <- vec_recycle_common(x = x, n = n)
+  x_precision <- calendar_precision_attribute(x)
+
+  n <- duration_collect_n(n, n_precision, error_call = error_call)
+
+  if (n_precision == PRECISION_QUARTER) {
+    n <- duration_cast(n, "month")
+    n_precision <- PRECISION_MONTH
+  }
+
+  size <- vec_size_common(x = x, n = n, .call = error_call)
+  args <- vec_recycle_common(x = x, n = n, .size = size)
   x <- args$x
   n <- args$n
 
   names <- names_common(x, n)
 
-  fields <- year_month_day_plus_duration_cpp(x, n, precision_fields, precision_n)
+  x <- vec_unstructure(x)
 
-  new_year_month_day_from_fields(fields, precision_fields, names = names)
+  if (n_precision == PRECISION_YEAR) {
+    fields <- year_month_day_plus_years_cpp(x$year, n)
+    x$year <- fields$year
+  } else if (n_precision == PRECISION_MONTH) {
+    fields <- year_month_day_plus_months_cpp(x$year, x$month, n)
+    x$year <- fields$year
+    x$month <- fields$month
+  } else {
+    abort("Unknown precision.", .internal = TRUE)
+  }
+
+  if (x_precision != n_precision) {
+    x <- df_list_propagate_missing(x, size = size)
+  }
+
+  new_year_month_day_from_fields(x, x_precision, names = names)
 }
 
 # ------------------------------------------------------------------------------
@@ -878,6 +996,8 @@ year_month_day_plus_duration <- function(x, n, precision_n) {
 #' Time points, Dates, POSIXct, and other calendars can all be converted to
 #' year-month-day.
 #'
+#' @inheritParams rlang::args_dots_empty
+#'
 #' @param x `[vector]`
 #'
 #'   A vector to convert to year-month-day.
@@ -893,32 +1013,35 @@ year_month_day_plus_duration <- function(x, n, precision_n) {
 #'
 #' # From other calendars
 #' as_year_month_day(year_quarter_day(2019, quarter = 2, day = 50))
-as_year_month_day <- function(x)  {
+as_year_month_day <- function(x, ...)  {
   UseMethod("as_year_month_day")
 }
 
 #' @export
-as_year_month_day.default <- function(x) {
+as_year_month_day.default <- function(x, ...) {
   stop_clock_unsupported_conversion(x, "clock_year_month_day")
 }
 
 #' @export
-as_year_month_day.clock_year_month_day <- function(x) {
+as_year_month_day.clock_year_month_day <- function(x, ...) {
+  check_dots_empty0(...)
   x
 }
 
 # ------------------------------------------------------------------------------
 
 #' @export
-as_sys_time.clock_year_month_day <- function(x) {
-  calendar_require_all_valid(x)
+as_sys_time.clock_year_month_day <- function(x, ...) {
+  check_dots_empty0(...)
+  calendar_check_no_invalid(x)
   precision <- calendar_precision_attribute(x)
   fields <- as_sys_time_year_month_day_cpp(x, precision)
   new_sys_time_from_fields(fields, precision, clock_rcrd_names(x))
 }
 
 #' @export
-as_naive_time.clock_year_month_day <- function(x) {
+as_naive_time.clock_year_month_day <- function(x, ...) {
+  check_dots_empty0(...)
   as_naive_time(as_sys_time(x))
 }
 
@@ -942,7 +1065,8 @@ calendar_month_factor.clock_year_month_day <- function(x,
                                                        ...,
                                                        labels = "en",
                                                        abbreviate = FALSE) {
-  calendar_month_factor_impl(x, labels, abbreviate, ...)
+  check_dots_empty0(...)
+  calendar_month_factor_impl(x, labels, abbreviate)
 }
 
 # ------------------------------------------------------------------------------
@@ -1007,7 +1131,8 @@ calendar_group.clock_year_month_day <- function(x, precision, ..., n = 1L) {
   n <- validate_calendar_group_n(n)
   x <- calendar_narrow(x, precision)
 
-  precision <- validate_precision_string(precision)
+  check_precision(precision)
+  precision <- precision_to_integer(precision)
 
   if (precision == PRECISION_YEAR) {
     value <- get_year(x)
@@ -1070,7 +1195,8 @@ calendar_group.clock_year_month_day <- function(x, precision, ..., n = 1L) {
 #' # narrowed to another subsecond precision
 #' try(calendar_narrow(micro, "millisecond"))
 calendar_narrow.clock_year_month_day <- function(x, precision) {
-  precision <- validate_precision_string(precision)
+  check_precision(precision)
+  precision <- precision_to_integer(precision)
 
   out_fields <- list()
   x_fields <- unclass(x)
@@ -1127,7 +1253,9 @@ calendar_narrow.clock_year_month_day <- function(x, precision) {
 #' try(calendar_widen(milli, "microsecond"))
 calendar_widen.clock_year_month_day <- function(x, precision) {
   x_precision <- calendar_precision_attribute(x)
-  precision <- validate_precision_string(precision)
+
+  check_precision(precision)
+  precision <- precision_to_integer(precision)
 
   if (precision >= PRECISION_MONTH && x_precision < PRECISION_MONTH) {
     x <- set_month(x, 1L)
@@ -1180,7 +1308,9 @@ NULL
 #' @export
 calendar_start.clock_year_month_day <- function(x, precision) {
   x_precision <- calendar_precision_attribute(x)
-  precision <- validate_precision_string(precision)
+
+  check_precision(precision)
+  precision <- precision_to_integer(precision)
 
   calendar_start_end_checks(x, x_precision, precision, "start")
 
@@ -1200,7 +1330,9 @@ calendar_start.clock_year_month_day <- function(x, precision) {
 #' @export
 calendar_end.clock_year_month_day <- function(x, precision) {
   x_precision <- calendar_precision_attribute(x)
-  precision <- validate_precision_string(precision)
+
+  check_precision(precision)
+  precision <- precision_to_integer(precision)
 
   calendar_start_end_checks(x, x_precision, precision, "end")
 
@@ -1269,7 +1401,8 @@ calendar_count_between.clock_year_month_day <- function(start,
 calendar_count_between_standardize_precision_n.clock_year_month_day <- function(x,
                                                                                 precision,
                                                                                 n) {
-  precision_int <- validate_precision_string(precision)
+  check_precision(precision)
+  precision_int <- precision_to_integer(precision)
 
   allowed_precisions <- c(PRECISION_YEAR, PRECISION_QUARTER, PRECISION_MONTH)
 
@@ -1289,7 +1422,8 @@ calendar_count_between_standardize_precision_n.clock_year_month_day <- function(
 calendar_count_between_compute.clock_year_month_day <- function(start,
                                                                 end,
                                                                 precision) {
-  precision <- validate_precision_string(precision)
+  check_precision(precision)
+  precision <- precision_to_integer(precision)
 
   if (precision == PRECISION_YEAR) {
     out <- get_year(end) - get_year(start)
@@ -1308,7 +1442,8 @@ calendar_count_between_compute.clock_year_month_day <- function(start,
 calendar_count_between_proxy_compare.clock_year_month_day <- function(start,
                                                                       end,
                                                                       precision) {
-  precision <- validate_precision_string(precision)
+  check_precision(precision)
+  precision <- precision_to_integer(precision)
 
   start <- vec_proxy_compare(start)
   end <- vec_proxy_compare(end)
@@ -1398,6 +1533,53 @@ seq.clock_year_month_day <- function(from,
 
 # ------------------------------------------------------------------------------
 
+#' @export
+clock_minimum.clock_year_month_day <- function(x) {
+  switch(
+    calendar_precision_attribute(x) + 1L,
+    clock_minimum_year_month_day_year,
+    abort("Invalid precision", .internal = TRUE),
+    clock_minimum_year_month_day_month,
+    abort("Invalid precision", .internal = TRUE),
+    clock_minimum_year_month_day_day,
+    clock_minimum_year_month_day_hour,
+    clock_minimum_year_month_day_minute,
+    clock_minimum_year_month_day_second,
+    clock_minimum_year_month_day_millisecond,
+    clock_minimum_year_month_day_microsecond,
+    clock_minimum_year_month_day_nanosecond,
+    abort("Invalid precision", .internal = TRUE)
+  )
+}
+
+#' @export
+clock_maximum.clock_year_month_day <- function(x) {
+  switch(
+    calendar_precision_attribute(x) + 1L,
+    clock_maximum_year_month_day_year,
+    abort("Invalid precision", .internal = TRUE),
+    clock_maximum_year_month_day_month,
+    abort("Invalid precision", .internal = TRUE),
+    clock_maximum_year_month_day_day,
+    clock_maximum_year_month_day_hour,
+    clock_maximum_year_month_day_minute,
+    clock_maximum_year_month_day_second,
+    clock_maximum_year_month_day_millisecond,
+    clock_maximum_year_month_day_microsecond,
+    clock_maximum_year_month_day_nanosecond,
+    abort("Invalid precision", .internal = TRUE)
+  )
+}
+
+year_month_day_minimum <- function(precision) {
+  calendar_minimum(precision, year_month_day(clock_calendar_year_minimum))
+}
+year_month_day_maximum <- function(precision) {
+  calendar_maximum(precision, year_month_day(clock_calendar_year_maximum))
+}
+
+# ------------------------------------------------------------------------------
+
 clock_init_year_month_day_utils <- function(env) {
   year <- year_month_day(integer())
 
@@ -1411,5 +1593,25 @@ clock_init_year_month_day_utils <- function(env) {
   assign("clock_empty_year_month_day_microsecond", calendar_widen(year, "microsecond"), envir = env)
   assign("clock_empty_year_month_day_nanosecond", calendar_widen(year, "nanosecond"), envir = env)
 
+  assign("clock_minimum_year_month_day_year", year_month_day_minimum("year"), envir = env)
+  assign("clock_minimum_year_month_day_month", year_month_day_minimum("month"), envir = env)
+  assign("clock_minimum_year_month_day_day", year_month_day_minimum("day"), envir = env)
+  assign("clock_minimum_year_month_day_hour", year_month_day_minimum("hour"), envir = env)
+  assign("clock_minimum_year_month_day_minute", year_month_day_minimum("minute"), envir = env)
+  assign("clock_minimum_year_month_day_second", year_month_day_minimum("second"), envir = env)
+  assign("clock_minimum_year_month_day_millisecond", year_month_day_minimum("millisecond"), envir = env)
+  assign("clock_minimum_year_month_day_microsecond", year_month_day_minimum("microsecond"), envir = env)
+  assign("clock_minimum_year_month_day_nanosecond", year_month_day_minimum("nanosecond"), envir = env)
+
+  assign("clock_maximum_year_month_day_year", year_month_day_maximum("year"), envir = env)
+  assign("clock_maximum_year_month_day_month", year_month_day_maximum("month"), envir = env)
+  assign("clock_maximum_year_month_day_day", year_month_day_maximum("day"), envir = env)
+  assign("clock_maximum_year_month_day_hour", year_month_day_maximum("hour"), envir = env)
+  assign("clock_maximum_year_month_day_minute", year_month_day_maximum("minute"), envir = env)
+  assign("clock_maximum_year_month_day_second", year_month_day_maximum("second"), envir = env)
+  assign("clock_maximum_year_month_day_millisecond", year_month_day_maximum("millisecond"), envir = env)
+  assign("clock_maximum_year_month_day_microsecond", year_month_day_maximum("microsecond"), envir = env)
+  assign("clock_maximum_year_month_day_nanosecond", year_month_day_maximum("nanosecond"), envir = env)
+
   invisible(NULL)
 }
